%%%%%%%%%%%%%%%%%%%%%% ch2ex.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Dec 1998
%----------------------------------------------------
%  This MATLAB file runs the 9 examples that appear 
%  in Chapter 2 (Laplace Transforms) in the BookWare 
%  Companion Feedback Control Problems Using MATLAB
%  by D. K. Frederick and J. H. Chow 
%	*************************************
%	*   requires the special functions: *
%	*   cpole2t.m, rpole2t.m 
%	*************************************
%------------------------------------------------------
clear, clf
axis('normal')  % do not use square aspect ratio for plots
%  menu to select Example to be run
example = -1;	% force menu to appear
while (example < 1) | (example > 9),
  disp(' 1 <= Example 2.1: Create G(s) as TF object')
  disp(' 2 <= Example 2.2: Create G(s) as ZPK object')
  disp(' 3 <= Example 2.3: Find PFE and impulse response')
  disp(' 4 <= Example 2.4: Response due to individual poles')
  disp(' 5 <= Example 2.5: Response due to repeated poles')
  disp(' 6 <= Example 2.6: Step response from G(s)')
  disp(' 7 <= Example 2.7: Response to pulse input')
  disp(' 8 <= Example 2.8: Poles and system stability')
  disp(' 9 <= Example 2.9: Blocking zeros')
  disp(' 10 <= Example 2.10: Right half-plane zero')
  example = input('Select an example, or enter 0 to return to chapter menu:  ');

if example == 0, disp('Completed Chapter 2 examples'), return, end

if example == 1, clf
  disp('Example 2.1: Find G(s), do PFE, plot g(t)')
  %   does PFE and zeros/poles of G(s) + impulse response
  disp(' ')
  disp('Create numerator & denominator polynomials of G(s)')
%  numG = [3 5 7]
%  denG = [1 3 4 2.5 0.8]
  numG = [4 36 32]
  denG = [1 10 30 40 24]
  disp('*****>');pause
  disp('Create G(s) as a TF object')
  G = tf(numG,denG)
  disp('*****>');pause
  disp('See properties of TF object')
  get(G)
  disp('*****>');pause
  disp('Extract numerator & denominator polynomials from TF object')
  [nn,dd] = tfdata(G,'v')
  disp('*****>');pause
  pzmap(G);grid
  title('Ex 2.1: pole-zero map from TF object')
  keyboard
  clear numG denG G nn dd
  disp('end of Example 2.1'); disp('  ')
  example =-1;	% force menu to appear
end	% of example 2.1 so return to menu
%-------------------------------------

if example == 2, clf
  disp('Example 2.2: Create G(s) as ZPK object')
  %   does ZP form of G(s) 
  n = [4 36 32]
  d = [1 10 30 40 24]
  disp('*****>');pause
  disp('Create G(s) as a TF object')
  G = tf(n,d)
  disp('*****>');pause
  disp('Convert G(s) into ZPK object')
  GG = zpk(G)
  disp('*****>');pause
  disp('Extract poles and zeros from ZP form into arrays')
  [zz,pp,kk] = zpkdata(GG,'v')
  disp('*****>');pause
  disp('Extract poles and zeros from TF form into arrays')
  [z,p,k] = zpkdata(G,'v')
  disp('*****>');pause
  pzmap(GG);grid
  title('Ex 2.2: pole-zero map from ZP form')
  keyboard
  clear n d G GG zz pp kk
  disp('end of Example 2.2'); disp('  ')
  example =-1;	% force menu to appear
end	% of Example 2.2 so return to menu
%-------------------------------------
if example == 3, clf
  disp('Example 2.3: Response due to individual poles')
  %   does PFE and zeros/poles of G(s) + impulse response
  %   Also does responses due to complex poles separately
  %--------------------------------------------------
  disp(' ')
  disp('Create numerator & denominator polynomials of G(s)')
  numG = [3 2]
  denG = [2 4 5 1]
  disp('*****>');pause
  disp('Create TF object of G(s)')
  G = tf(numG,denG)
  disp('*****>');pause
  disp('Determine the zeros, poles, and gain of G(s)')
  [zG,pG,kG] = zpkdata(G,'v')
  disp('*****>');pause
  disp('Do partial-fraction expansion to get residues at each pole')
  disp('Note: the residue function accepts polynomial coefficients only')
  [resG,polG,otherG] = residue(numG,denG)
  disp('*****>');pause
  impulse(G); grid
  %title('Ex 2.3 - impulse response');
  keyboard
  clear numG denG zG pG kG resG polG otherG 
  disp('end of Example 2.3'); disp('  ')
  example =-1;	% force menu to appear
end	% of Example 2.3 so return to menu
%-------------------------------------
if example == 4, clf
  disp('Example 2.4: Responses due to distinct poles')
  %   does steps of Example 2.3 for system with 
  %	 repeated poles (file=EE)
  %--------------------------------------------------
  disp(' ')
  disp('Create numerator & denominator polynomials of G(s)')
  numG = [3 2]
  denG = [2 4 5 1]
  disp('*****>');pause
  disp('Do partial-fraction expansion to get residues at each pole')
  [resG,polG,otherG] = residue(numG,denG)
  disp('*****>');pause
  disp('Solve for the contributions of the individual poles')
  disp('   and plot them with the system''s impulse response')
  t = [0:0.1:20];
  %%%-- response due to complex poles
  ycmplx = cpole2t(polG(1),resG(1),t);
  yreal = rpole2t(polG(3),resG(3),t);
  ytot = ycmplx + yreal;
  plot(t,ytot,t,ycmplx,t,yreal,'--');grid
  title('Ex 2.4: responses: total, & from complex & real poles')
  xlabel('Time (secs)');ylabel('Amplitude')
  text(2.0,0.6,'total')
  text(4.0,0.16,'real pole')
  text(4.0,-0.06,'complex poles')
  keyboard
  disp('Plot the impulse response and the sum of the individual')
  disp('   responses to verify that they are the same')
  disp('*****>');pause
  G = tf(numG,denG)
  t1 = [0:0.5:20]';
  yy = impulse(G,t1);
  plot(t,ytot), grid, hold on
  title('Ex 2.4: sum of responses')
  disp('*****>');pause
  plot(t1,yy,'o'), hold off
  title('Ex 2.4: sum of responses (solid) and g(t) (circle) agree exactly')
  xlabel('Time (seconds)')
  keyboard
  clear numG denG zG pG kG resG polG otherG G
  clear ycmplx yreal t ytot yy
  disp('end of Example 2.4'); disp('  ')
  example =-1;	% force menu to appear
end	% of Example 2.4 so return to menu
%-------------------------------------

if example == 5, clf
  disp('Example 2.5: Responses due to repeated poles')
  %   does steps of Example 2.2 for system with 
  %	 repeated poles (file=EE)
  %--------------------------------------------------
  disp(' ')
  disp('Create numerator & denominator polynomials of G(s)')
  numG = [1 2]
  denG = [1 1 0.25]
  disp('*****>');pause
  disp('Do partial-fraction expansion to get residues at each pole')
  [resG,polG,otherG] = residue(numG,denG)
  disp('*****>');pause
  disp('Calculate contributions of each term in the PFE and plot them')
  t = [0:0.1:20];
  %%%- do contribution of each term
  g1 = rpole2t(polG(1),resG(1),t);
  %%%%--- do element-by-element multiplication
  g2 = t.*rpole2t(polG(2),resG(2),t);
  plot(t,g1,t,g2);grid
  %  student - need to label individual plots
  title('Ex 2.5: Contributions of each term in the PFE')
  xlabel('Time (seconds)')
  keyboard
  disp('Plot the impulse response and the sum of the individual')
  disp('   responses to verify that they are the same')
  disp('*****>');pause
  G = tf(numG,denG);
  t1 = [0:0.5:20]';
  y = impulse(G,t1);
  plot(t,g1+g2), grid, hold on
  title('Ex 2.5: sum of responses')
  disp('*****>');pause
  plot(t1,y,'o'), hold off
  title('Ex 2.5: sum of responses (solid) and g(t) (circle) agree exactly')
  xlabel('Time (seconds)')
  keyboard
  clear numG denG resG polG otherG t  g1 g2 y G
  disp('end of Example 2.5'); disp('  ')
  example =-1;	% force menu to appear
end	% of Example 2.5 so return to menu
%-------------------------------------

if example == 6, clf
  disp('Example 2.6: Step response from G(s)')
  %   given G(s), find step response
  disp('  ')
  disp('Create numerator & denominator polynomials of G(s)')
  numG = [3 2]	
  denG = [2 4 5 1]
  disp('*****>');pause
  disp('Add pole at s=0 to G(s) to get transfer function of step response')
  numstep = numG
  denstep = [denG 0]
  disp('*****>');pause
  disp('Compute and plot step response for G(s) by finding the')
  disp('   impulse response of G(s) + extra pole at s = 0')
  Gstep = tf(numstep,denstep)
  impulse(Gstep,20)
  % student - need explanation on why title says impulse response
  grid;
  %title('Ex 2.6: Step response by adding pole to G(s)')
  keyboard
  disp('Get step response via step command')
  G = tf(numG,denG)
  step(G,20)
  % student - put note on to say that this is generated from step
  grid;
  %title('Ex 2.6: Step response via step cmd')
  keyboard
  disp('calculate DC gain from G(s) as ratio of')
  disp('......constant terms in numerator & denominator')
  DC_gain = dcgain(G)
  disp('*******>');pause
  clear numG denG numstep denstep G Gstep
  disp('end of Example 2.6'); disp('  ')
  example =-1;	% force menu to appear
end	% of Example 2.6 so return to menu
%-------------------------------------

if example == 7, clf
  disp('Example 2.7: Response to pulse input')
  %   given G(s), find response to arbitrary inputs via lsim
  %--------------------------------------------------
  disp(' ')
  disp('Create TF form of G(s) using numerator & denominator polynomials')
  disp(' directly as input arguments.')
  G = tf([3 2],[2 4 5 1])
  disp('Construct a time vector, the input, and use the lsim')
  disp('   command to solve for the response')
  disp('*****>');pause
  time = [0:0.02:10]';
  u = 2.0*(1+0*(time)); % vector of 2's with length of "time"
  for ii=min(find(time>=2.0)):length(u), u(ii) = 0.5;end
  y = lsim(G,u,time);
  plot(time,y,time,u);grid
  title('Ex 2.7: zero-state response to pulse input')
  xlabel('Time (seconds)');ylabel('Amplitude')
  text(3.5,1.7,'output')
  text(4.0,0.58,'input')
  keyboard
  clear numG denG time u y G
  disp('end of Example 2.7'); disp('  ')
  example =-1;	% force menu to appear
end	% of Example 2.7 so return to menu
%-------------------------------------

if example == 8, clf
  disp(' ')
  disp('Example 2.8: Poles and system stability')
  disp(' ')
  numG = [1.5 1]
  denG = [1 2 2.5 0.5]
  G = tf(numG,denG)
  disp('*****>');pause
  disp('poles determined as roots of denominator')
  pGroots = roots(denG)
  disp('poles determined from ''pole'' command')
  pG = pole(G)
  disp('System is stable because all the poles are in the')
  disp('  left-half plane. The impulse-response plot shows that ')
  disp('  the impulse response decays to zero.')
  disp('*****>');pause
  pzmap(G);grid
  title('Ex 2.8: pole-zero map')
  keyboard
  impulse(G)
  grid, %title('Ex 2.8 : impulse response')
  keyboard
  clear numG denG G
  disp('end of Example 2.8')
  example =-1;	% force menu to appear
end	% of example 2.8 so return to menu

%-------------------------------------

if example == 9, clf
  disp(' ')
  disp('Example 2.9: Blocking zeros')
  disp('  This example shows that the complex zeros of a ')
  disp('  transfer function block the transmission of a ')
  disp('  sinusoidal signal.')
  disp(' ')
  zz = [-5; i; -i]	% generate G(s) in ZP form
  pp = [-1; -2; -2; -3]
  kk = 1
  G = zpk(zz,pp,kk)
  disp('*****>');pause
  pzmap(G);grid
  axis([-5 1 -2 2])	% get complex zeros off plot boundary
  title('Ex 2.9: pole-zero map')
  disp('zeros determined as roots of numerator polynomial')
  [numG,denG] = tfdata(G,'v');
  zGroots = roots(numG)
  disp('zeros determined by the ''tzero'' command')
  zG = tzero(G)
  disp('The system has complex zeros at +/-j1. the following ')
  disp('  plot shows the response of the system to a sinusoidal ')
  disp('  signal with a frequency of 1 rad/s. Note the ')
  disp('  absence of the sinosoidal components in the response.')
  keyboard
  t = [0:0.1:10];
  e = sin(t);
  lsim(G,e,t)
  grid, %title('Ex 2.9 : time response')
  keyboard
  disp('We can factor out the complex zeros from numG and ')
  disp('  perform an impluse response simulation. Note that ')
  disp('  the resulting response is identical to the response ')
  disp('  shown in the previous plot.')
  disp('*****>');pause
  [numG,denG] = tfdata(G,'v');
  G1 = tf([1 5],denG);
  impulse(G1,t)
  grid, %title('Ex 2.9 : impulse response with zeros factored out')
  % student - do the last two plots as a single superimposed plot. 
  %  as in example 2.5. 
  keyboard
  clear numG denG zz pp kk e t G
  disp('end of Example 2.9')
  example =-1;	% force menu to appear
end	% of Example 2.9 so return to menu
%
%-------------------------------------

if example == 10, clf
  disp(' ')
  disp('Example 2.10: Right half-plane zero')
  disp('  This example shows that the a zero of H(s)')
  disp('  in the right half-plane can have a step response')
  disp('  that starts out in the negative direction.')
  disp('  It uses the model of the hydro-turbine system.')
  disp(' ')
  numG = [-2 1]
  denG = [0.5 1.5 1]
  G = tf(numG,denG)
  disp('*****>');pause
  [zz,pp,kk] = zpkdata(G,'v')
  disp('The system has a right half-plane zero at s = +0.5')
  disp('*****>');pause
  step(G)
  grid, %title('Ex 2.10 : Step response with a zero at s=+0.5')
  keyboard
  clear numG denG zz pp kk e t G
  disp('end of Example 2.10')
  example =-1;	% force menu to appear
end	% of Example 2.10 so return to menu
%
%-----------
end		% of while
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
