%%%%%%%%%%%%%%%%%%%%%% cp8_2.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 6/14/99 HS
%----------------------------------------------------
%	     		  Comprehensive Problem CP8.2
%   TF model of satellite (zero initial wheel speed)
%    Use pole with PD to avoid improper TFs
%      Does root locus & ref step response
%----------------------------------------------------
disp('CP8.2: TF model of satellite (zero initial wheel speed)')
clear, clf
% load sat.mat if exists, otherwise quit this M-file
if chkmat('sat') == 1, 
    load sat.mat
  else
     return
end
numGp = numG1; denGp = denG1; % rename TF for plant
Gp = tf(numGp,denGp)

disp('Book suggests KD values in 5 <= KD <= 50')
KD = input('enter derivative gain KD ==> ');
KD_str=num2str(KD);
T = 0.05*KD;	% add pole to avoid improper Gc(s)
T_str = num2str(T);
%-- TF of PD controller w/o gain, but require "-"
disp('TF of PD controller w/o gain, but with negative sign')
numXX = -[(KD+T) 1];  
denXX = [T 1];	     % denominator "  "     "
G_XX = tf(numXX,denXX)
%---------- open-loop transfer function
XGp = G_XX*Gp
[numXGp,denXGp] = tfdata(XGp,'v')
disp('*********>');pause
rlocus(XGp);grid
title('CP8.2: Satellite, open-loop, PD control')
text('Units','Norm','Pos',...
		[0.8 0.3],'String',['KD = ' KD_str])
text('Units','Norm','Pos',...
		[0.8 0.2],'String',['T = ' T_str])
keyboard
[kk,polesCL] = rlocfind(XGp)
keyboard
%
%----- step response with gain sweep to select KP
t = [0:1:200];
denGcGp = denXGp;	% same denominator
%------------- closed-loop step response to reference input
for KP = [-0.0005:-0.0005:-0.0020];
  numGc = KP*[(KD+T) 1];	% numerator of PD controller
  numGcGp = conv(numGc,numGp);
  GcGp = tf(numGcGp,denGcGp)
  GcGp_CL = feedback(GcGp,1,-1);
  y = step(GcGp_CL,t);
  plot(t,y)
  KP
  disp('*****>'),pause
  if ishold ~=  1, hold on, end
end
xlabel('Time (s)'),ylabel('output')
grid;title('CP8.2b: Satellite TF model CL step response, PD')
text('Units','Norm','Pos',[0.4 0.4],...
     'String','KP = -0.0005, -0.0015,..., -0.0020')
text('Units','Norm','Pos',[0.4 0.3],'String',['KD = ' KD_str])
text('Units','Norm','Pos',[0.4 0.2],'String',['T = ' T_str])
hold off
%------ single step response with specific KP
KP = input('enter controller gain KP (< 0 for stability) ==> ');
KP_str=num2str(KP);
t = [0:0.5:200];
%------------- closed-loop step response to reference input
numGc = KP*[(KD+T) 1];	% numerator of PD controller
numGcGp = conv(numGc,numGp);
GcGp = tf(numGcGp,denGcGp)
GcGp_CL = feedback(GcGp,1,-1);
y = step(GcGp_CL,t);
plot(t,y)
xlabel('Time (s)'),ylabel('output')
grid;title('CP8.2: Satellite TF model CL step resp, PD ctrl')
text('Units','Norm','Pos',[0.5 0.4],...
     'String',['KP = ' KP_str])
text('Units','Norm','Pos',[0.5 0.3],'String',['KD = ' KD_str])
text('Units','Norm','Pos',[0.5 0.2],'String',['T = ' T_str])
[Mo,tp,tr,ts,ess] = tstats(t,y,1)
disp('end of Comprehensive Problem 8.2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

