%%%%%%%%%%%%%%%%%%%%%% ex9_1.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%  last revision: 1/8/99  JHC
%----------------------------------------------------
clear,clf
disp('Ex. 9.1: Lag control via Franklin''s algorithm')
%----- plant & sensor transfer functions
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4; 
Gp = tf(Kproc,conv([tauP1 1],[tauP2 1]));
H  = tf(1,[tau_sen 1]);
%---------- open-loop transfer function
GpH = Gp*H;
%-------------- open-loop frequency response
w = logspace(-1,1,100)';
[mag,ph] = bode(GpH,w);
%-------- display table of phase, mag, & freq values
mag = reshape(mag,100,1);
ph  = reshape(ph ,100,1);
disp('index  phase     magnitude    freq  ')  
for i = find((ph <= -100)& (ph >= -120)),
   disp([ i  ph(i)  mag(i)  w(i)])
end
disp('*********>');pause
%--- interpolate to get magnitude for desired phase margin
disp('want 70 deg pm so interpolate at ph = -110 deg')
mag110 = interp1(ph,mag,-110)
Klag = 1/mag110
Klag_str = num2str(Klag);
lfg = Klag*dcgain(GpH);
Alag = 49/lfg
Alag_str = num2str(Alag);
disp('*********>');pause
disp('Specify value for zero corner freq in interval ')
disp('      wpm/10 < zero corner freq < wpm/2')
disp('   where wpm is the phase-margin frequency without lag')
disp('Book uses zero corner freq = 0.25 rad/s')
w_Zlag = input('Enter corner freq for lag zero ==> ')
Zlag = -w_Zlag
Zlag_str=num2str(Zlag);
%------------- build the lag then use
%		forward transfer function is Gc * Gp
disp('controller TF:')
Gc = tf(Klag*[1 -Zlag],[1 -Zlag/Alag])
GcGpH = Gc*GpH
%------- compute low-freq gain of current design
disp('low-freq gain of current design is:')
lfg = dcgain(GcGpH)
disp('*********>');pause
%
%-------------- open-loop frequency response
[km,pm,wkm,wpm] = margin(GcGpH);
disp('gain margin in dB & omega_gm are:')
disp([20*log10(km)  wkm])
disp('phase margin in degrees & omega_pm are:')
disp([pm wpm])
disp('*********>');pause
%---- do the plot
margin(GcGpH)
text(0.01,-200,['Klag = ' Klag_str])
text(0.01,-240,['Zlag = ' Zlag_str])
text(0.01,-280,['Alag = ' Alag_str])
keyboard
%==========  OL frequency responses for Fig. 9.3 =====
w = logspace(-3,2,100);
ph180 = -180*ones(length(w));
%---- first, with gain only
[mag_K,ph_K] = bode(Klag*GpH,w);
mag_K = reshape(mag_K,100,1);
ph_K  = reshape(ph_K,100,1);   
mag_db_K = 20*log10(mag_K);
%-------- then with final lag design
[mag_lag,ph_lag] = bode(GcGpH,w);
mag_lag = reshape(mag_lag,100,1);
ph_lag  = reshape(ph_lag, 100,1);
mag_db_lag = 20*log10(mag_lag);
%--------- now plot both sets
subplot(211)
semilogx(w,mag_db_lag,'-',w,mag_db_K,'--');grid;
xlabel('Frequency (rad/s)')
ylabel('Magnitude (dB)')
title('Ex 9.1: OL freq responses, per Fig. 9.3')
subplot(212)
semilogx(w,ph_lag,'-',w,ph_K,'--',w,ph180,':');grid
xlabel('Frequency (rad/s)')
ylabel('Phase angle (deg)')
keyboard
subplot(111)	% reset plot window
%
%==========  CL step responses for Fig. 9.4 =====
%---  reference input: fwd path = Gc*Gp;  fdbk path = H
T_ref = feedback(Gp*Gc,H,-1)
t = [0:0.02:10];
y_ref = step(T_ref,t);
%=========== repeat for disturbance input ===========
%--- dist input: fwd path = Gp; fdbk path = -H*Gc
%------ closed-loop step response to disturbance input
%--- use +1 for feedback gain because neg sign is in H(s)
T_dist = feedback(Gp,-Gc*H,+1)
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
T_dist = feedback(Gp,-Gc*H,+1)
T_dist = feedback(Gp,-Gc*H,+1)
grid;
title('Ex 9.1: CL ref & dist step responses, Fig. 9.4')
text(6.0,0.7,['Klag = ' Klag_str])
text(6.0,0.6,['Zlag = ' Zlag_str])
text(6.0,0.5,['Alag = ' Alag_str])
disp('end of Example Problem 9.1')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

