%%%%%%%%%%%%%%%%%%%%%% ex9_3.m %%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, Jan. 99
%----------------------------------------------------
clear,clf
disp('Lead control')
%----- plant & sensor parameters
tauP1 = 2; tauP2 = 0.5; tau_sen = 0.05; Kproc = 4;
Gp = tf(Kproc,conv([tauP1 1],[tauP2 1]))
H  = tf(1,[tau_sen 1])
%---------- open-loop transfer function
GpH = Gp*H
%
%===== Part(a) determine Klead & phase margin for gain-only
K_reqd = (1/0.04) - 1     % required gain to give 4% ess
plant_lfg = dcgain(GpH)
Klead = K_reqd/plant_lfg
disp('*********>');pause
Klead_str = num2str(Klead);
[km,pm,wkm,wpm] = margin(Klead*GpH) % pm with gain only
disp('*********>');pause
%====== Part (b) determine lead alpha
w = logspace(-1,2,100);
[mag,ph] = bode(Klead*GpH,w);
mag = reshape(mag,100,1);
ph  = reshape(ph, 100,1);
pm_target = 85	% phase-margin target in degrees
del_pm = pm_target - pm        % max phase angle in deg 
Alead = (1+sin(del_pm*pi/180))/(1-sin(del_pm*pi/180)) % Eq (9.2)
Alead_str = num2str(Alead);
disp('*********>');pause
%====== Part (c) determine lead zero
wc = interp1(mag,w,1/sqrt(Alead)) % use (9.4) to find wc 
Zlead = -wc/sqrt(Alead)      % using (9.3)
Zlead_str = num2str(Zlead);
disp('*********>');pause
Gc = tf(Klead*Alead*[1  -Zlead],[1  -Alead*Zlead]) % use (9.1) for lead TF
%---- open-loop transfer function of compensated system
GcGpH = Gc*GpH;
%------- compute low-freq gain of current design
disp('low-freq gain of current design is:')
lfg = dcgain(GcGpH)
%
%==========  OL frequency responses for Fig. 9.8 =====
w = logspace(-2,3,100);
ph180 = -180*ones(length(w));
%---- first, with gain only
[mag_K,ph_K] = bode(Klead*GpH,w);
mag_K = reshape(mag_K,100,1);
ph_K  = reshape(ph_K, 100,1);
mag_db_K = 20*log10(mag_K);
%-------- then with final lead design
[mag_lead,ph_lead] = bode(GcGpH,w);
mag_lead = reshape(mag_lead,100,1);
ph_lead  = reshape(ph_lead, 100,1);
mag_db_lead = 20*log10(mag_lead);
%--------- now plot both sets
subplot(211)
semilogx(w,mag_db_lead,'-',w,mag_db_K,'--');grid;
xlabel('Frequency (rad/s)')
ylabel('Magnitude (dB)')
text(100,-25,'Lead')
text(3,-65,'Proportional')
title('Ex 9.3: OL freq response per Fig. 9.8')
subplot(212)
semilogx(w,ph_lead,'-',w,ph_K,'--',w,ph180,'-');grid
text(20,-130,'Lead')
text(1.0,-240,'Proportional')
xlabel('Frequency (rad/s)')
ylabel('Phase angle (deg)')
keyboard
subplot(111)	% reset plot window

%==========  CL step responses for Fig. 9.9 =====
%---  reference input: fwd path = Gc*Gp;  fdbk path = H
T_ref = feedback(Gp*Gc,H,-1)
disp('*********>');pause
t = [0:0.005:2];
y_ref = step(T_ref,t);

%=========== repeat for disturbance input ===========
%--- dist input: fwd path = Gp; fdbk path = -H*Gc
T_dist = feedback(Gp,-Gc*H,+1)
disp('*********>');pause
y_dist = step(T_dist,t);
%-------- plot both responses
plot(t,y_ref,t,y_dist,'--')
xlabel('Time (sec.)'),ylabel('Amplitude')
grid;
title('Ex 9.3: CL ref & dist step responses Fig. 9.9')
text(1.0,0.60,['Klead = ' Klead_str])
text(1.0,0.50,['Zlead = ' Zlead_str])
text(1.0,0.40,['Alead = ' Alead_str])
text(1.0,1.00,'reference')
text(0.7,0.19,'disturbance')
axis([0 2 0 1.2])
disp('end of Example Problem 9.3')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
